import games
import util


class NimGameState(games.GameState):
    """
        Nim game states. 
        
        All you need to know about this class is that it implements the GameState interface,
        and it stores its board in gameState.board as a list of heap sizes       
    """
    
    actionCalls = 0
    successorCalls = 0
    def __init__(self, board = [7,7,7], players = ("1", "2"), previousMove = None):
        self.players = players
        self.board = board 
        self.previousMove = previousMove

    def getPlayers(self):
        """  Returns a tuple of player names, where the first player plays next. """
        return self.players

    def getPreviousMove(self):
        """  Returns the most recently made move. """
        return self.previousMove


    def actions(self):
        """  Return a list of actions possible from this state. """
        NimGameState.actionCalls += 1
        actions = []
        for i, row in enumerate(self.board):
            for x in range(row):
                actions.append((i, x+1))
        return actions
      
        
    def successor(self, action):
        """  Return the successor state of self on action.  
             Precondition: action in actions(self) """
        NimGameState.successorCalls += 1
        (row,amount) = action
        newBoard=self.board[:]
        newBoard[row] -= amount
        return NimGameState(newBoard, util.leftRotate(self.players), action)
            
    
    def utility(self):
        """  Returns None if the game is nonterminal, or a Dictionary 
             associating players to utilities if it is terminal. """
        (currentPlayer, otherPlayer) = self.players
        
        for row in self.board: 
            if row > 0:
                return None
            
        return {self.players[0] : -1, self.players[1] : 1}
        
        

    def __str__(self):
        """  Return a printable string representation of this state. """
        return "\n".join(["Heap %d: %s" % (i, "*"*x) for i, x in enumerate(self.board)])

    def representation(self):
        """  Return a hashable representation of self that will be 
             used for representation and equality tests.  """
        return "%s%s" % (self.board, self.players)


    # These methods are used to evaluate SearchAgents    
    def getSearchStats(self):
        """ return (# actions calls, # successor calls) """
        return (NimGameState.actionCalls, NimGameState.successorCalls)

    
    def resetSearchStats(self):
        """ Reset internal state of problem for tracking search statistics. """    
        NimGameState.actionCalls = 0
        NimGameState.successorCalls = 0
